@@ -1,9 +1,9 @@
 /**
  * Javascript to add events to form elements
  *
- * $Horde: horde/js/form_helpers.js,v 1.3.4.3 2005/07/06 01:53:30 chuck Exp $
+ * $Horde: horde/js/form_helpers.js,v 1.3.4.7 2007/01/02 13:55:04 jan Exp $
  *
- * Copyright 2004-2005 Matt Kynaston <matt@kynx.org>
+ * Copyright 2004-2007 Matt Kynaston <matt@kynx.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -34,15 +34,15 @@ function addEvent(element, event, code)
     } else if (element.attachEvent) {
         element.attachEvent(event, code);
     } else if (element.onload != null) {
-        eval('var OldEvent = element.' + event);
+        var OldEvent = element[event];
         newCode = Function(e)
         {
             oldEvent(e);
             code();
         };
-        eval('element.' + event + ' = newCode');
+        element[event] = newCode;
     } else {
-        eval('element.' + event + ' = code');
+        element[event] = code;
     }
 
     return true;
@@ -91,7 +91,7 @@ function checkEnabled()
             case 'select-multiple' :
                 val = new Array();
                 count = 0;
-                for (i = 0; i < objSrc.length; i++) {
+                for (var i = 0; i < objSrc.length; ++i) {
                     if (objSrc.options[i].selected) {
                         val[count] = objSrc.options[i].value;
                     }
@@ -109,7 +109,7 @@ function checkEnabled()
                 val = objSrc.value;
             }
 
-            for (i = 3; i < arguments.length; i++) {
+            for (var i = 3; i < arguments.length; ++i) {
                 if (typeof(val) == 'object' && (arguments[i] in val)) {
                     toggle = true;
                     break;
@@ -144,7 +144,7 @@ function sumFields()
         objTarget = objFrm.elements[arguments[1]];
         sum = 0;
         if (objTarget) {
-            for (i = 2; i < arguments.length; i++) {
+            for (var i = 2; i < arguments.length; ++i) {
                 objSrc = objFrm.elements[arguments[i]];
                 if (objSrc) {
                     switch (objSrc.type.toLowerCase()) {
@@ -153,7 +153,7 @@ function sumFields()
                         break;
 
                     case 'select-multiple' :
-                        for (j = 0; j < objSrc.length; j++) {
+                        for (var j = 0; j < objSrc.length; ++j) {
                             sum += toNumber(objSrc.options[j].value);
                         }
                         break;
