@@ -24,7 +24,7 @@
  * The table structure for the DataTree system is in
  * scripts/sql/horde_datatree.sql.
  *
- * $Horde: framework/DataTree/DataTree/sql.php,v 1.156.2.32 2007/01/02 13:54:13 jan Exp $
+ * $Horde: framework/DataTree/DataTree/sql.php,v 1.156.2.33 2007/07/14 13:06:45 jan Exp $
  *
  * Copyright 1999-2007 Stephane Huther <shuther@bigfoot.com>
  * Copyright 2001-2007 Chuck Hagenbuch <chuck@horde.org>
@@ -1460,6 +1460,16 @@ class DataTree_sql extends DataTree {
                 $values = array_merge($values, $binds[1]);
             } else {
                 if (isset($vals['field'])) {
+                    // All of the attribute_* fields are text, so make
+                    // sure we send strings to the database.
+                    if (is_array($vals['test'])) {
+                        for ($i = 0, $iC = count($vals['test']); $i < $iC; ++$i) {
+                            $vals['test'][$i] = (string)$vals['test'][$i];
+                        }
+                    } else {
+                        $vals['test'] = (string)$vals['test'];
+                    }
+
                     $binds = Horde_SQL::buildClause($this->_db, $alias . '.attribute_' . $vals['field'], $vals['op'], $vals['test'], true);
                     $clause .= $binds[0];
                     $values = array_merge($values, $binds[1]);
