@@ -11,7 +11,7 @@ include_once 'Horde/String.php';
 /**
  * Class representing iCalendar files.
  *
- * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.45 2007/03/14 15:58:24 jan Exp $
+ * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.46 2007/03/15 18:36:27 jan Exp $
  *
  * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
  *
@@ -923,25 +923,6 @@ class Horde_iCalendar {
             case 'RRULE':
                 break;
 
-            // Attendance
-            case 'ATTENDEE':
-                // Kronolith creates attendee field in vcalendar2.0 format.
-                // Convert to vcalendar1.0 if necessary.
-                // Example of 1.0 style:
-                // ATTENDEE;ROLE=OWNER;STATUS=CONFIRMED:John Smith <jsmith@host1.com>
-                if ($this->isOldFormat()) {
-                    $value = preg_replace('/MAILTO:/i', '', $value);
-                    $value =  $value . ' <' . $value . '>';
-                    $params_str = str_replace('PARTSTAT=', 'STATUS=', $params_str);
-                    $params_str = str_replace('STATUS=NEEDS-ACTION', 'STATUS=NEEDS ACTION', $params_str);
-                    $params_str = str_replace('ROLE=REQ-PARTICIPANT', 'EXPECT=REQUIRE', $params_str);
-                    $params_str = str_replace('ROLE=OPT-PARTICIPANT', 'EXPECT=REQUEST', $params_str);
-                    $params_str = str_replace('ROLE=NON-PARTICIPANT', 'EXPECT=FYI', $params_str);
-                    $params_str = str_replace('RSVP=TRUE', 'RSVP=YES', $params_str);
-                    $params_str = str_replace('RSVP=FALSE', 'RSVP=NO', $params_str);
-                }
-                break;
-
             default:
                 if ($this->isOldFormat()) {
                     if (is_array($attribute['values']) &&
@@ -959,11 +940,6 @@ class Horde_iCalendar {
                          * semicolons */
                         $value = str_replace(';', '\\;', $value);
                     }
-                    if ($name == 'STATUS') {
-                        // vcalendar 1.0 has STATUS:NEEDS ACTION while 2.0 has
-                        // STATUS:NEEDS-ACTION.
-                        $value = str_replace('NEEDS-ACTION', 'NEEDS ACTION', $value);
-                    }
                     // Text containing newlines or ASCII >= 127 must be BASE64
                     // or QUOTED-PRINTABLE encoded. Currently we use
                     // QUOTED-PRINTABLE as default.
